/**
 * \file    BSplineFloorIssue.cxx
 * \author  Daniel B. Russakoff
 *
 * \modifications
 * 	-2007/09/13 DBR, Created.
 * 
 */

#include "itkImage.h"
#include "itkBSplineDeformableTransform.h"
#include <iomanip>

int main( int argc, char *argv[] )
{

// BSplineFloor Issue 
	double d;
	d = 12.9;		std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.99;		std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.999;		std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.9999;	std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.99999;	std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.999999;	std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.9999999; std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.99999999; std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.999999999; std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.9999999999; std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;
	d = 12.99999999999; std::cout << std::setprecision(12) << d <<": "<<BSplineFloor(d) << " vs. "<< floor(d) << std::endl;


// BSplineFloor Issue leading to a crash
    const    unsigned int    ImageDimension = 3;
    typedef  short           PixelType;
    const unsigned int SpaceDimension = ImageDimension;
    const unsigned int SplineOrder = 3;
    typedef double CoordinateRepType;

    typedef itk::BSplineDeformableTransform<
                            CoordinateRepType,
                            SpaceDimension,
                            SplineOrder >      TransformType;
	typedef TransformType::RegionType		   RegionType;
    typedef itk::Image< short, ImageDimension> ImageType;
    typedef TransformType::SpacingType	       SpacingType;
    typedef TransformType::OriginType		   OriginType;
    typedef TransformType::ParametersType      ParametersType;

	RegionType bsplineRegion;

	RegionType::SizeType    gridSizeOnImage;
	RegionType::SizeType    gridBorderSize;
    RegionType::SizeType    totalGridSize;
    gridBorderSize.Fill( 3 );    // Border for spline order = 3 ( 1 lower, 2 upper )
	gridSizeOnImage.Fill( 12 );
	totalGridSize = gridSizeOnImage + gridBorderSize;
    bsplineRegion.SetSize( totalGridSize );

	ImageType::SizeType imageSize;
	imageSize[0] = 254; imageSize[1] = 254; imageSize[2] = 183;

    SpacingType spacing;
	spacing[0] =  11.2305; spacing[1] =  11.2305; spacing[2] =  10.0;
	
	OriginType  origin;
	origin[0] = 79.1015; origin[1] = 16.1133; origin[2] = 25.625;

    TransformType::Pointer  transform = TransformType::New();
    transform->SetGridSpacing( spacing );
    transform->SetGridOrigin( origin );
    transform->SetGridRegion( bsplineRegion );

    ParametersType parameters( transform->GetNumberOfParameters() );
    parameters.Fill( 0.0 );

	transform->SetParameters( parameters );

	std::cout << transform << std::endl;

    ImageType::PointType point;

	std::cout << "point = "<<point<<std::endl;

	point[0] = 207.71411662649854;
	point[1] = 162.10979098954641;
	point[2] = 125.68023428572201;

	try {
		transform->TransformPoint( point);
	} catch( itk::ExceptionObject & err ) { 
	    std::cerr << "ExceptionObject caught !" << std::endl; 
		std::cerr << err << std::endl; 
		return -1;
    } 

    return 0;
}
